'use strict';

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

require('babel/polyfill');

require('source-map-support/register');

var _proxyquire = require('proxyquire');

var _proxyquire2 = _interopRequireDefault(_proxyquire);

var _chai = require('chai');

var _redux = require('redux');

var realRedux = _interopRequireWildcard(_redux);

_proxyquire2['default'].noPreserveCache();

var combinedReducers = [];
var reducers = undefined;

function createStore(reducer) {
  reducers = [reducer];
  return {
    isStore: true,
    replaceReducer: function replaceReducer(r) {
      if (typeof r !== 'function') {
        throw new Error('Reducer must be a function');
      }
      reducers.push(r);
    }
  };
}

function populateReducers(redux) {
  var num = arguments.length <= 1 || arguments[1] === undefined ? 3 : arguments[1];

  for (var i = 0; i < num; i++) {
    redux.addReducer('key' + i, function (state, action) {
      return state;
    });
  }
}

// This has to be done to test if the reducer is changed on storeSet. Otherwise,
// combineReduces is called again, and a new instance is returned, which !==
var combinedReducer = function combinedReducer(state, action) {
  return state;
};

describe('modular-redux', function () {
  describe('reducers', function () {
    var redux = undefined;
    beforeEach(function () {
      combinedReducers = [];
      redux = (0, _proxyquire2['default'])('./index', {
        redux: {
          createStore: createStore,
          combineReducers: function combineReducers(rs) {
            combinedReducers.push(rs);
            return combinedReducer;
          }
        }
      });
    });

    it('should automatically create a store', function () {
      (0, _chai.expect)(redux.getStore().isStore).to.eql(true);
    });

    it('should update the store\'s reducer whenever a reducer is added', function () {
      populateReducers(redux);
      (0, _chai.expect)(reducers).to.have.lengthOf(4); // Inital reducer is included;
    });

    it('getReducer should return the store\'s (current) reducer', function () {
      (0, _chai.expect)(redux.getReducer()).to.eql(reducers[0]);
      populateReducers(redux);
      (0, _chai.expect)(redux.getReducer()).to.equal(reducers.pop());
    });

    it('should combine reducers', function () {
      populateReducers(redux);
      (0, _chai.expect)(Object.keys(combinedReducers.pop())).to.have.lengthOf(3);
    });

    it('when given a new store, it should set the reducer', function () {
      populateReducers(redux);
      var expectedReducer = reducers.pop();

      redux.setStore(createStore(function (state, action) {
        return state;
      }));

      (0, _chai.expect)(reducers).to.have.lengthOf(2);
      (0, _chai.expect)(reducers.pop()).to.eql(expectedReducer);
    });

    it('if given a new store, getStore should return it', function () {
      var store = createStore(function (state, action) {
        return state;
      });
      redux.setStore(store);
      (0, _chai.expect)(redux.getStore()).to.eql(store);
    });

    it('should correctly handle object paths', function () {
      var r = function r(state, action) {
        return state;
      };
      redux.addReducer('obj.paths.are.cool', r);
      var rs = combinedReducers.pop();
      (0, _chai.expect)(function () {
        return rs.obj.paths.are.cool;
      }).to.not['throw'](); // in case it's
      // undefined
      (0, _chai.expect)(rs.obj.paths.are.cool).to.eql(r);
    });
  });

  describe('ActionTypes', function () {
    var redux = undefined;
    beforeEach(function () {
      redux = (0, _proxyquire2['default'])('./index', {});
    });

    it('.types, .actions, .actionTypes, and, .ActionTypes should all be equal', function () {
      (0, _chai.expect)(redux.ActionTypes).to.eql(redux.types);
      (0, _chai.expect)(redux.ActionTypes).to.eql(redux.actions);
      (0, _chai.expect)(redux.ActionTypes).to.eql(redux.actionTypes);
    });

    it('should add a property to .types with key when addActionType is called', function () {
      redux.addActionType('ACTION_1', 'ACTION_1');
      (0, _chai.expect)(redux.types.ACTION_1).to.exist;
    });

    it('the value should default to the key', function () {
      redux.addActionType('ACTION_1');
      (0, _chai.expect)(redux.types.ACTION_1).to.eql('ACTION_1');
    });

    it('should convert the key to uppercase, and spaces to _s', function () {
      redux.addActionType('action 1');
      (0, _chai.expect)(redux.types.ACTION_1).to.exist;
      (0, _chai.expect)(redux.types['action 1']).to.not.exist;
    });

    it('should convert *not* convert the value to uppercase', function () {
      redux.addActionType('action 1');
      (0, _chai.expect)(redux.types.ACTION_1).to.eql('action 1');
    });

    it('should allow completly diffrent keys and values', function () {
      redux.addActionType('ACTION_1', 'foo');
      (0, _chai.expect)(redux.types.ACTION_1).to.eql('foo');
    });

    it('only allow strings as keys or values', function () {
      (0, _chai.expect)(function () {
        return redux.addActionType({ 'foo': 'bar' }, 'baz');
      }).to['throw']();
      (0, _chai.expect)(function () {
        return redux.addActionType('FOO', 3);
      }).to['throw']();
      (0, _chai.expect)(function () {
        return redux.addActionType('foo bar', false);
      }).to['throw']();
      (0, _chai.expect)(function () {
        return redux.addActionType('foo bar baz', null);
      }).to['throw']();
    });

    it('should export addActionType as .addType and .ActionType', function () {
      (0, _chai.expect)(redux.addActionType).to.eql(redux.addType);
      (0, _chai.expect)(redux.addActionType).to.eql(redux.ActionType);
    });
  });

  describe('ActionCreators', function () {
    var redux = undefined;
    beforeEach(function () {
      redux = (0, _proxyquire2['default'])('./index', {});
    });

    it('.ActionCreators, .creators and .create should all be the same', function () {
      (0, _chai.expect)(redux.ActionCreators).to.eql(redux.creators);
      (0, _chai.expect)(redux.ActionCreators).to.eql(redux.create);
    });

    it('should put the creator on .ActionTypes.<key to camelcase>', function () {
      var creator = function creator() {
        return { type: 'something happened', payload: 'OK' };
      };
      redux.addActionType('something happened', 'something happened', creator);
      (0, _chai.expect)(redux.ActionCreators.somethingHappened).to.exist;
      (0, _chai.expect)(redux.ActionCreators.somethingHappened).to.eql(creator);
    });

    it('should handle addActionType(key, constructor)', function () {
      var creator = function creator() {
        return { type: 'FOO_BAR' };
      };
      redux.addActionType('FOO_BAR', creator);
      (0, _chai.expect)(redux.ActionTypes.FOO_BAR).to.eql('FOO_BAR');
      (0, _chai.expect)(redux.ActionCreators.fooBar).to.eql(creator);
    });

    it('should default to a creator that returns { type: type }', function () {
      redux.addActionType('FOO_BAR');
      (0, _chai.expect)(redux.ActionCreators.fooBar()).to.deep.equal({ type: 'FOO_BAR' });

      redux.addActionType('FOO_BAR_BAZ', 'qux quux');
      (0, _chai.expect)(redux.ActionCreators.fooBarBaz()).to.deep.equal({ type: 'qux quux' });
    });

    it('each ActionCreator should have a .bound property', function () {
      redux.addActionType('FOO_BAR_BAZ_QUX');
      (0, _chai.expect)(redux.ActionCreators.fooBarBazQux.bound).to.exist;

      var dispatched = [];

      // We need at least 1 reducer or redux complains
      redux.addReducer('foo', function (state, action) {
        if (state === undefined) state = 0;
        return state;
      });

      var store = redux.getStore();
      store.dispatch = function (action) {
        return dispatched.push(action);
      };
      redux.setStore(store);

      redux.ActionCreators.fooBarBazQux.bound();

      (0, _chai.expect)(dispatched[0]).to.deep.equal({ type: 'FOO_BAR_BAZ_QUX' });
    });
  });

  it('should export all of redux\'s exports', function () {
    var _expect$to;

    var redux = (0, _proxyquire2['default'])('./index', {});

    var filterExports = function filterExports(e) {
      return e.filter(function (k) {
        return k.charAt(0) !== '_';
      });
    };

    var realReduxExports = filterExports(Object.keys(realRedux));
    var modularReduxExports = filterExports(Object.keys(redux));

    (_expect$to = (0, _chai.expect)(modularReduxExports).to).include.apply(_expect$to, _toConsumableArray(realReduxExports));
  });
});